/*.............................................................

Silverlight, Enterprise Class Server Room Monitor

Author: NLD

Date: 2019/Sep/12

Version: 0.2

This code might not compile for other arduino models than the Pro-Micro.

Tools>Board>Sparkfun ProMicro
Processor: Atmega32U4 (5V 16Mhz)

Changes:
v0.2:
-lcd texts replaced by symbols
-final pinout
-delay removed from motion detection
v0.1:
-This code uses 1 way communication Ardu -> PC (Status Report)
-It pulls all sensors repeatedly and displays accurate results in the LCD screen
but it only sends data to the pc in every 10 seconds not to flood the serial port.
..............................................................*/
#include <LiquidCrystal.h>
#include "DHT.h"

/* Temperature Sensor */
#define DHTPIN A0
#define DHTTYPE DHT22
DHT dht(DHTPIN, DHTTYPE);

/* LCD Screen Settings */
const int LCD_NB_ROWS = 2;
const int LCD_NB_COLUMNS = 16;
const int LCD_R = 4; // BackLight - R
const int LCD_G = 5; // BackLight - G
const int LCD_B = 6; // BackLight - B

//RS, Enable, D4, D5, D6, D7
LiquidCrystal lcd(2, 3, 15, 14, 16, 10);

/* Other IO */
#define SMOKE_SENSOR A3
#define MOTION_SENSOR A2
#define PHOTOCELL A1
#define LED_RED 7
#define LED_GREEN 8
#define SPEAKER 9

/* LCD led pins are inverted (they are active when they are pulled down to GND) */
int ON=HIGH;
int OFF=LOW;
int LON=LOW;
int LOFF=HIGH;

char Msg[100];

uint32_t curr_time;
uint32_t last_event;
bool on_hold=false;
uint32_t hold_time=10000; // 10 seconds print hold 

byte ico_tmp[8] =
{
  B00100,
  B01010,
  B01010,
  B01110,
  B01110,
  B11111,
  B11111,
  B01110
};

byte ico_hum[8] =
{
  B00100,
  B00100,
  B01010,
  B01010,
  B10001,
  B10001,
  B10001,
  B01110
};

byte ico_sol[8] =
{
  0B00000,
  0B00000,
  0B10101,
  0B01110,
  0B11111,
  0B01110,
  0B10101,
  0B00000
}; 

byte ico_smk[8] = {
  B00010,
  B00000,
  B00100,
  B01110,
  B01110,
  B11111,
  B11111,
  B01110
};

/* 1 -> Red, 2 -> Blue, 3 -> Green */
void color(int c)
{
        if (c==1)
        {
          digitalWrite(LCD_R, LON);
          digitalWrite(LCD_G, LOFF);
          digitalWrite(LCD_B, LOFF);   
        }
        else if (c==2)
        {
          digitalWrite(LCD_R, LOFF);
          digitalWrite(LCD_G, LON);
          digitalWrite(LCD_B, LOFF); 
        }
        else if (c==3)
        {
          digitalWrite(LCD_R, LOFF);
          digitalWrite(LCD_G, LOFF);
          digitalWrite(LCD_B, LON);   
        }    
}
void setup() {

 pinMode(LCD_R, OUTPUT); // BackLight - R
 pinMode(LCD_G, OUTPUT); // BackLight - G
 pinMode(LCD_B, OUTPUT); // BackLight - B    
 lcd.begin(LCD_NB_COLUMNS, LCD_NB_ROWS); 
 lcd.createChar(1, ico_tmp);
 lcd.createChar(2, ico_hum);
 lcd.createChar(3, ico_sol);
 lcd.createChar(4, ico_smk);
 lcd.clear();


 pinMode(SPEAKER, OUTPUT);
 pinMode(LED_GREEN, OUTPUT);
 pinMode(LED_RED, OUTPUT); 

 digitalWrite(LED_RED, LOW);
 digitalWrite(LED_GREEN, LOW);

 Serial.begin(9600);  // USB

/* Print LCD intro */

  color(1);
  lcd.setCursor(0, 0);  
  lcd.print("Silverlight by");
  lcd.setCursor(0, 1);  
  lcd.print("NLD standby...");  
  delay(20000);  // MQ2 + PIR (Motion) sensor warm up time
  lcd.clear();
}  

void loop() {

 curr_time = millis();

  // Limit events
  if (on_hold && curr_time - last_event >= hold_time) {
        on_hold = false;
    }
    
  // Read in all sensor data in every loop
  int humidity = dht.readHumidity();
  int temp = dht.readTemperature();
  int f = dht.readTemperature(true);
  int photocellReading = analogRead(PHOTOCELL); 
  int motion_status = digitalRead(MOTION_SENSOR);
  int smoke = analogRead(SMOKE_SENSOR);
   
  int hi_f = dht.computeHeatIndex(f,humidity); //heat index in fahrenheit
  int heat_index  = (hi_f-32)*5/9; // convert heat index to celcius

/* LED status indicators */
if (motion_status == HIGH)
{
 digitalWrite(LED_GREEN, HIGH);
// Serial.println("Motion detected");
// delay(5000);
}
else
{
 digitalWrite(LED_GREEN, LOW);
// Serial.println("No motion :P");
// delay(5000);
}

if (smoke > 50)
{
 digitalWrite(LED_RED, HIGH);
 tone(SPEAKER, 1000, 200);
}
else
{
 digitalWrite(LED_RED, LOW);
 noTone(SPEAKER);
}

//This delay is necessary otherwise the LCD would be flickering like crazy from the constant changes
delay(1000);

/* Display what is possible on the LCD  */
lcd.setCursor(0, 0);  
lcd.write(1);
lcd.setCursor(2, 0);  
lcd.print(temp);
lcd.print("    ");
lcd.setCursor(5, 0);  
lcd.write(0b11011111);
lcd.print("C");
lcd.setCursor(9, 0);
lcd.write(2);
lcd.print(" ");
lcd.print(humidity);
lcd.print("    ");
lcd.setCursor(14, 0);  
lcd.print("%");

lcd.setCursor(0, 1);  
lcd.write(3);
lcd.print(" ");
lcd.print(photocellReading);
lcd.print("    ");
lcd.setCursor(9, 1);  
lcd.write(4);
lcd.print(" ");
lcd.print(smoke);
lcd.print("    ");

/* Change the LCD backlight color */
        if (temp <= 20)
        {
          color(3);
        }
        else if (temp > 20 && temp <= 27)
        {
          color(2);
        }
        else if (temp > 27)
        {
          color(1);
        }
  
/* Send the message for the PC and lock for 10 seconds */
if (!on_hold)
 {
  //Order kept the same as in Nat weather station for simplicity, motion + smoke added to the end  
  sprintf(Msg, "%d,%d,%d,%d,%d,%d", humidity, temp, heat_index, photocellReading, motion_status, smoke);
  Serial.println(Msg);
  last_event = curr_time;
  on_hold = true;
 }                 

}
